import dao.*
import dao.old.Items
import org.jetbrains.exposed.sql.and
import org.jetbrains.exposed.sql.deleteAll
import org.jetbrains.exposed.sql.selectAll
import org.jetbrains.exposed.sql.transactions.transaction
import org.jetbrains.exposed.sql.update

class MigrationsController {
    private val migrations: Map<String, () -> Boolean> = mapOf(
        "move-items-to-new-table" to {moveItemsToNewTable()},
        "unchecked-position-change" to {updatePositionForUnchecked()}
    )

    fun checkMigrationsToDo() {
        this.migrations.forEach {
            transaction {
                val migrationWithName = Migration.find { Migrations.identifier eq it.key }.firstOrNull()
                if (migrationWithName == null) {
                    Migration.new {
                        identifier = it.key
                    }
                    println("Migration ${it.key} will start now.")
                    it.value()
                    Migrations.update({Migrations.identifier eq it.key}) {
                        it[done] = true
                    }
                    println("Migration ${it.key} done.")
                } else if (!migrationWithName.done) {
                    println("Migration ${it.key} not done yet. Redoing.")
                    it.value()
                    Migrations.update({Migrations.identifier eq it.key}) {
                        it[done] = true
                    }
                    println("Migration ${it.key} done.")
                } else {
                    println("Migration ${it.key} done.")
                }
            }
        }
    }

    private fun moveItemsToNewTable() : Boolean {
        try {
            transaction {
                Items.selectAll().forEach {
                    ItemV2.new {
                        list = it[Items.list]
                        content = it[Items.content]
                        position = if (it[Items.checked]) null else it[Items.position]
                        checked = it[Items.checked]
                    }
                }
                Items.deleteAll()
            }
        } catch (e: Exception) {
            return false
        }
        return true
    }

    private fun updatePositionForUnchecked() : Boolean {
        try {
            transaction {
                Lists.selectAll().forEach {
                    var i = 0
                    ItemV2.find { ItemsV2.list eq it[Lists.id].value and (ItemsV2.checked eq false) }
                        .sortedBy { it.position }
                        .forEach {
                            it.position = i
                            i++
                        }
                }
            }
        } catch (e: Exception) {
            return false
        }
        return true
    }
}