package dao

import io.javalin.http.Context
import org.jetbrains.exposed.sql.and
import org.jetbrains.exposed.sql.transactions.transaction

class DB {

    /*
    * Lists
    */
    fun createList(c: ListView, ctx: Context) {
        transaction {
            var i = List.new {
                name = c.name
            }
            ctx.json(i.toView())
        }

    }

    fun deleteList(resourceId: Int, ctx: Context) {
        transaction {
            val list = List.findById(resourceId)

            if (list != null) {
                ItemV2.find { ItemsV2.list eq list.id.value}.forEach { i -> i.delete() }
            }
            
            list?.delete() ?: ctx.status(404)
        }
    }

    fun getAllLists(ctx: Context) {
        transaction {
            ctx.json(List.all().map { it.toView() })
        }
    }

    fun findList(resourceID: Int, ctx: Context) {
        transaction {
            val listOpt = List.findById(resourceID)
            if (listOpt != null) {
                ctx.json(listOpt.toView())
            } else {
                ctx.status(404)
            }
        }
    }

    fun updateList(resourceID: Int, body: ListView, ctx: Context) {
        transaction {
            val list = List.findById(resourceID)
            if (list != null) {
                if (body.name != null) {
                    list?.name = body.name
                }
                ctx.json(list.toView())
            } else {
                ctx.status(404)
            }
        }
    }

    /*
    * Items
    */
    fun findListItems(resourceID: Int, ctx: Context) {
        transaction {
            val listOpt = List.findById(resourceID)
            if (listOpt != null) {
                val items = ItemV2.find { ItemsV2.list eq resourceID }
                    .sortedBy { it.position }

                ctx.json(listOpt.toViewWithItems(items))
            } else {
                ctx.status(404)
            }
        }
    }

    fun createItem(listId: Int, c: ItemView, ctx: Context) {
        transaction {
            val listOpt = List.findById(listId)
            if (listOpt != null) {
                val count = ItemV2.find { ItemsV2.list eq listId and  (ItemsV2.checked eq false) }.count()
                val p = (count + 1)

                var i = ItemV2.new {
                    list = listId
                    content = c.content
                    position = p.toInt()
                    checked = false
                }
                
                ctx.json(i.toView())
            } else {
                ctx.status(404)
            }
        }

    }

    private fun reorderElements(listId: Int, position: Int?, itemId: Int? = null) {
        transaction {
            var i = 0
            ItemV2.find { ItemsV2.list eq listId and (ItemsV2.id neq itemId  and  (ItemsV2.checked eq false)) }
                .sortedBy { it.position }
                .forEach {
                    if (position != null && i == position) {
                        i++
                    }
                    if (it.position != null) {
                        it.position = i
                        i++
                    }
                }
        }
    }

    fun deleteItem(listId: Int, itemId: Int, ctx: Context) {
        transaction {
            val listOpt = List.findById(listId)
            if (listOpt != null) {
                val item = ItemV2.find { ItemsV2.list eq listId and (ItemsV2.id eq itemId) }.limit(1).firstOrNull()
                if (item != null) {
                    item.delete()
                    var i = 0
                    ItemV2.all()
                        .sortedBy { it.position }
                        .forEach {
                            it.position = i
                            i++
                        }
                } else {
                    ctx.status(404)
                }
            } else {
                ctx.status(404)
            }
        }
    }

    fun updateItem(listId: Int, itemId: Int, body: ItemPatchView, ctx: Context) {
        transaction {
            val listOpt = List.findById(listId)
            if (listOpt != null) {
                val item = ItemV2.find { ItemsV2.list eq listId and (ItemsV2.id eq itemId) }.limit(1).firstOrNull()
                if (item != null) {
                    if (body.position != null) {
                        reorderElements(listId, body.position, itemId)
                        item.position = body.position
                    } 
                    if (body.checked != null) {
                        var p: Int? = if (body.checked) {
                            null
                        } else {
                            (ItemV2.find { ItemsV2.list eq listId and  (ItemsV2.checked eq false) }.count() + 1).toInt()
                        }
                        item.position = p
                        reorderElements(listId, p, itemId)
                        item.checked = body.checked
                    }
                    if (body.content != null) {
                        item.content = body.content
                    }
                    ctx.json(item.toView())
                } else {
                    ctx.status(404)
                }
            } else {
                ctx.status(404)
            }
        }
    }

    /*
    * Api keys
    */
    fun createKey(ctx: Context) {
        transaction {
            val allowedChars = ('A'..'Z') + ('a'..'z') + ('0'..'9')
            val k = (1..256)
                .map { allowedChars.random() }
                .joinToString("")
            var key = Key.new {
                value = k
            }
            ctx.json(key.toView())
        }
    }

    fun checkKey(ctx: Context) {
        transaction {
            val key = ctx.body()
            if (Key.find { Keys.value eq key }.count() > 0L) ctx.status(301) else ctx.status(404)

        }
    }

    fun deleteKey(ctx: Context) {
        transaction {
            val k = ctx.body<KeyView>()
            val key = Key.find { Keys.value eq k.value }.firstOrNull()
            key?.delete() ?: ctx.status(404)
        }
    }
}